#include <string.h>
#include "libavformat/avformat.h"
#include "input.h"

static uint64_t time_convert(uint64_t time, AVStream *st)
{
    if (st->start_time != AV_NOPTS_VALUE) {
#if 1
        time -= st->start_time;
#else
fprintf(stderr, "%Ld\t%Ld\n", st->start_time, av_rescale_q(st->start_time,
                                  st->time_base, AV_TIME_BASE_Q));
#endif
    }
    time = av_rescale_q(time, st->time_base, AV_TIME_BASE_Q);

    return time;
}

static void time_base_convert(AVPacket *pkt, AVStream *st)
{
  if (pkt->pts != AV_NOPTS_VALUE) {
    pkt->pts = time_convert(pkt->pts, st);
  }
  if (pkt->dts != AV_NOPTS_VALUE) {
    pkt->dts = time_convert(pkt->dts, st);
  }
  if (pkt->duration != 0) {
    pkt->duration = av_rescale_q(pkt->duration, st->time_base, AV_TIME_BASE_Q);
  }
}

AVFormatContext *open_input_stream(const char *fname, int w, int h, int fps)
{
    AVFormatContext *s;
    AVInputFormat *fmt;
    AVFormatParameters param;
    int res;

    memset(&param, 0, sizeof(AVFormatParameters));
    /* FIXME: Set these!!! */
    param.width  = w;
    param.height = h;
    param.pix_fmt = PIX_FMT_YUV420P;
    param.time_base.den = fps;
    param.time_base.num = 1;
    fmt = av_find_input_format("video4linux2");
    res = av_open_input_file(&s, fname, fmt, 0, &param);
    if (res < 0) {
        fprintf(stderr, "Error opening %s: %d\n", fname, res);

        return NULL;
    }

    res = av_find_stream_info(s);
    if (res < 0) {
        fprintf(stderr, "Cannot find codec parameters for %s\n", fname);

        return NULL;
    }

    dump_format(s, 0, fname, 0);

    return s;
}

void close_input_stream(AVFormatContext *s)
{
    av_close_input_file(s);
}

AVPacket *read_input_packet(AVFormatContext *s)
{
    static AVPacket pkt;
    AVStream *st;
    int res;

    res = av_read_frame(s, &pkt);
    if (res < 0) {
        return NULL;
    }

    st = s->streams[pkt.stream_index];
    if (pkt.dts == AV_NOPTS_VALUE) {
        fprintf(stderr, "WTF??? Unknown DTS???\n");

	return NULL;
    }

    time_base_convert(&pkt, st);

    return &pkt;
}
