#include "libavformat/avformat.h"


AVFormatContext *open_output_stream(const char *dst, int port, enum CodecType codec_type)
{
    AVFormatContext *s;
    static AVOutputFormat *rtp_format;
    AVStream *st;

    s = av_alloc_format_context();
    rtp_format = guess_format("rtp", NULL, NULL);
    if (!rtp_format) {
        fprintf(stderr, "Unable for find the RTP format for output\n");

        return NULL;
    }

    s->oformat = rtp_format;
    st = av_new_stream(s, 0);
    if (!st) {
        fprintf(stderr, "Cannot allocate stream\n");

        return NULL;
    }

    snprintf(s->filename, sizeof(s->filename), "rtp://%s:%d", dst, port);
        
    /* open the UDP sockets for RTP and RTCP */
    if (url_fopen(&s->pb, s->filename, URL_WRONLY) < 0) {
        fprintf(stderr, "Cannot open '%s'\n", s->filename);
        /* FIXME: Free the stream! */

        return NULL;
    }
    avcodec_get_context_defaults2(s->streams[0]->codec, codec_type);
    s->streams[0]->codec->codec_id = av_guess_codec(s->oformat, NULL, s->filename, NULL, codec_type);
    s->streams[0]->codec->codec_id = CODEC_ID_MPEG4;
    s->streams[0]->codec->pix_fmt  = PIX_FMT_YUV420P;

    return s;
}

int pkt_send(AVFormatContext *ctx, AVPacket *pkt)
{
  static int inited;		/* FIXME: HACK! */


  if (!inited) {
    int res;

fprintf(stderr,"initing!\n");
    res = av_write_header(ctx);
    if (res < 0) {
      fprintf(stderr, "Cannot open the out RTP stream!\n");

      return res;
    }
    inited = 1;
  }

  return av_write_frame(ctx, pkt);
}

